#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import subprocess


class Version:
    def __init__(self, frozen):
        self._get_hash(frozen)
        self._get_string(frozen)
        self._get_url(frozen)
        self._get_token(frozen)
        self._get_text(frozen)

    def _get_hash(self, frozen):
        if frozen:
            self.hash = frozen.hg_hash
        else:
            path = os.path.dirname(os.path.dirname(__file__))
            try:
                pipe = subprocess.Popen(['hg', 'id', '-i', '-R', path], stdout=subprocess.PIPE)
                self.hash = pipe.stdout.read().decode()
            except OSError:
                self.hash = 'unknown'
        self.hash = self.hash.strip()

    def _get_string(self, frozen):
        if frozen:
            self.string = frozen.version
        else:
            self.string = '0.0.0'

    def _get_url(self, frozen):
        if frozen:
            self.url = frozen.url
        else:
            self.url = ''

    def _get_token(self, frozen):
        if frozen:
            self.token = frozen.token
        else:
            self.token = ''

    def _get_text(self, frozen):
        if frozen:
            self.text = frozen.text
        else:
            self.text = ''



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import datetime
from PyQt5 import QtCore, QtWidgets
import requests


class WUpdates(QtWidgets.QMessageBox):
    def __init__(self, parent, version):
        super().__init__(parent)
        self.version = version
        self.threadUpdateChecker = None
        self.setupUI()
        self.deleteThreadUpdateChecker()

    def setupUI(self):
        self.setWindowTitle('Check updates')
        self.setStandardButtons(self.Ok)
        self.cb = QtWidgets.QCheckBox('Do not check updates automatically')
        self.setCheckBox(self.cb)

    def error(self, text):
        self.setIcon(self.Critical)
        self.setText('There has been an error during update checking')
        self.setInformativeText(text)
        self.exec()

    def success(self, main, info):
        self.setIcon(self.Information)
        if info:
            self.setText(main)
            self.setInformativeText(info)
        else:
            self.setText('There are no new updates')
        self.exec()

    def loadSettings(self):
        s = QtCore.QSettings()
        self.cb.setChecked(s.value('Version/doNotCheckUpdates', False, bool))

    def saveSettings(self):
        s = QtCore.QSettings()
        s.setValue('Version/doNotCheckUpdates', self.cb.isChecked())

    def checkNewVersion(self):
        if not self.version.url:
            return
        if not self.cb.isChecked():
            self.createUpdateChecker()
            self.threadUpdateChecker.start()

    def checkNewVersionByUser(self):
        if not self.version.url:
            return
        self.createUpdateChecker()
        self.checkUpdatesByUser = True
        self.threadUpdateChecker.start()

    def showNewVersion(self, text, error):
        if self.checkUpdatesByUser:
            self.error(text) if error else self.success(self.version.text, text)
        else:
            if not error and not self.cb.isChecked() and text:
                self.success(self.version.text, text)

    def createUpdateChecker(self):
        self.checkUpdatesByUser = False
        self.threadUpdateChecker = QtCore.QThread()
        self.updateChecker = UpdateChecker()
        self.updateChecker.moveToThread(self.threadUpdateChecker)
        # noinspection PyUnresolvedReferences
        self.threadUpdateChecker.started.connect(lambda: self.updateChecker.check(self.version))
        self.updateChecker.sigNewVersion.connect(self.showNewVersion)
        self.updateChecker.sigFinished.connect(self.deleteThreadUpdateChecker)

    def deleteThreadUpdateChecker(self):
        self.checkUpdatesByUser = False
        if self.threadUpdateChecker:
            self.threadUpdateChecker.quit()
            self.threadUpdateChecker.wait()
        self.threadUpdateChecker = None
        self.updateChecker = None


class UpdateChecker(QtCore.QObject):
    sigNewVersion = QtCore.pyqtSignal(str, bool)
    sigFinished = QtCore.pyqtSignal()

    def check(self, version):
        try:
            r = requests.get(version.url)
        except requests.RequestException as err:
            self.sigNewVersion.emit(str(err), True)
        else:
            self.parse(r.text, version)
        self.sigFinished.emit()

    def parse(self, text, version):
        current = datetime.datetime(*[int(i) for i in version.string.split('.')])
        strings = []
        put = False
        boldline = False
        first = True
        parsing = False
        for line in text.split('\n'):
            if not line:
                continue
            if line == version.token:
                parsing = True
                continue
            if parsing:
                if line.startswith('..'):
                    continue
                items = line.split()
                try:
                    newv = [int(i) for i in items[1].split('.')]
                except (IndexError, ValueError):
                    pass
                else:
                    try:
                        put = datetime.datetime(*newv) > current
                    except ValueError:
                        pass
                    boldline = True
                if put:
                    line = line.strip()
                    line = line[line.index(" ") + 1:]
                    if boldline:
                        boldline = False
                        if not first:
                            strings.append('</ul>')
                        strings.append(f'<span style="font-weight: bold; color: red">{line}</span>'
                                       f'<ul style="list-style-type:disc">')
                    else:
                        strings.append(f'<li>{line}</li>')
                    first = False
        if strings:
            strings.append('</ul>')
        self.sigNewVersion.emit('\n'.join(strings), False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import subprocess


DirUI = 'resources'
DirStore = 'qtsnbl'


def compile_ui():
    for fl in os.listdir(DirUI):
        if fl.endswith('.ui'):
            prog = 'pyuic5 --from-imports'
            out = f'{fl[:-2]}py'
        elif fl.endswith('.qrc'):
            prog = 'pyrcc5'
            out = f'{fl[:-4]}.py'
        else:
            prog = None
        if prog:
            s = f'{prog} {DirUI}/{fl} > {DirStore}/{out}'
            print(s)
            subprocess.run(s, shell=True)


if __name__ == '__main__':
    compile_ui()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from PyQt5 import QtCore
import pyqtgraph as pg


class AxisItem(pg.AxisItem):
    sigChangeRange = QtCore.pyqtSignal(float, float)

    def __init__(self, *args, **kwargs):
        self.range = [0, 1]
        super().__init__(*args, **kwargs)
        self.real_range = [0, 1]

    def setRange(self, mn, mx):
        self.real_range = [mn, mx]
        self.sigChangeRange.emit(mn, mx)

    def setRealRange(self, mn, mx):
        return super().setRange(mn, mx)

    def changeUnits(self):
        self.setRange(*self.real_range)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
from setuptools import setup


install_requires = [
    'numpy',
    'pyqtgraph',
    'requests',
]
if '-gentoo' not in os.uname().release:
    install_requires.append('PyQt5')


setup(
    name='qtsnbl',
    version='2017.11.20',
    description='PyQt widgets for SNBL programs',
    author='Vadim Dyadkin',
    author_email='dyadkin@gmail.com',
    url='https://hg.3lp.cx/qtsnbl',
    license='GPLv3',
    install_requires=install_requires,
    packages=[
        'qtsnbl',
    ],
    classifiers=[
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
    ],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# noinspection PyUnresolvedReferences
from . import resources
# noinspection PyUnresolvedReferences
from .fixedwidget import FixedWidget
# noinspection PyUnresolvedReferences
from .imageview import ImageView
# noinspection PyUnresolvedReferences
from .axisitem import AxisItem
# noinspection PyUnresolvedReferences
from .viewbox import ViewBox
# noinspection PyUnresolvedReferences
from .cutslicewidget import CutSliceWidget
# noinspection PyUnresolvedReferences
from .updater import WUpdates



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import json
from PyQt5 import QtCore, QtWidgets, QtGui
import numpy as np
import pyqtgraph as pg
from pyqtgraph import dockarea
from .imageview import ImageView


class CutSliceWidget(QtWidgets.QWidget):
    SizeX = 1600
    SizeY = 1200
    Size1D = 200
    sig2DMouseLeftClicked = QtCore.pyqtSignal(float, float)
    sigXMouseMoved = QtCore.pyqtSignal(float, float)
    sigYMouseMoved = QtCore.pyqtSignal(float, float)

    def __init__(self, parent=None, *, imageview=None):
        super().__init__(parent)
        self.resize(self.SizeX, self.SizeY)
        self.plot2DView = imageview or ImageView()
        self.setActions()
        self.setUI()
        self.connectSignals()

    # noinspection PyUnresolvedReferences
    def connectSignals(self):
        self.actionAdjustLevels.triggered.connect(self.adjustImageLevels)
        self.actionRotateImage.triggered.connect(self.plot2DView.rotateImage)
        self.actionFlipImageH.triggered.connect(self.plot2DView.flipHImage)
        self.actionFlipImageV.triggered.connect(self.plot2DView.flipVImage)
        self.actionLogarithmicScale.triggered.connect(self.plot2DView.logImage)
        self.plot2DView.sigHCutValues.connect(self.plotHCut)
        self.plot2DView.sigVCutValues.connect(self.plotVCut)
        self.plot2DView.sigMouseLeftClicked.connect(self.sig2DMouseLeftClicked.emit)

    def setActions(self):
        self.actionRotateImage = QtWidgets.QAction(self)
        self.actionRotateImage.setIcon(QtGui.QIcon(':/rotate'))
        self.actionRotateImage.setText('Rotate image clockwise')
        self.actionRotateImage.setToolTip('Rotate image clockwise')
        self.actionRotateImage.setShortcut('Ctrl+R')
        self.actionFlipImageH = QtWidgets.QAction(self)
        self.actionFlipImageH.setIcon(QtGui.QIcon(':/flip-horizontal'))
        self.actionFlipImageH.setText('Flip image horizontally')
        self.actionFlipImageH.setToolTip('Flip image horizontally')
        self.actionFlipImageH.setShortcut('Ctrl+H')
        self.actionFlipImageV = QtWidgets.QAction(self)
        self.actionFlipImageV.setIcon(QtGui.QIcon(':/flip-vertical'))
        self.actionFlipImageV.setText('Flip image vertically')
        self.actionFlipImageV.setToolTip('Flip image vertically')
        self.actionFlipImageV.setShortcut('Ctrl+V')
        self.actionAdjustLevels = QtWidgets.QAction(self)
        self.actionAdjustLevels.setIcon(QtGui.QIcon(':/adjust'))
        self.actionAdjustLevels.setText('Adjust image levels')
        self.actionAdjustLevels.setToolTip('Adjust image levels')
        self.actionAdjustLevels.setShortcut('Ctrl+A')
        self.actionLogarithmicScale = QtWidgets.QAction(self)
        self.actionLogarithmicScale.setCheckable(True)
        self.actionLogarithmicScale.setIcon(QtGui.QIcon(':/exp'))
        self.actionLogarithmicScale.setText('Set logarithmic scale')
        self.actionLogarithmicScale.setToolTip('Set logarithmic scale')
        self.actionLogarithmicScale.setShortcut('Ctrl+L')

    def setUI(self):
        self.docks = dockarea.DockArea()
        self.dock2D = dockarea.Dock('2D', size=(self.SizeX-self.Size1D, self.SizeY-self.Size1D))
        self.dockx = dockarea.Dock('x', size=(self.SizeX, self.Size1D))
        self.docky = dockarea.Dock('y', size=(self.Size1D, self.SizeY))
        self.docks.addDock(self.dock2D, 'left')
        self.docks.addDock(self.dockx, 'bottom', self.dock2D)
        self.docks.addDock(self.docky, 'right', self.dock2D)
        self.toolbar2D = QtWidgets.QToolBar(self.dock2D)
        self.toolbar2D.addActions([w for a, w in self.__dict__.items() if a.startswith('action')])
        self.dock2D.addWidget(self.toolbar2D)
        self.dock2D.addWidget(self.plot2DView)
        self.plotx = pg.PlotWidget(self.dockx)
        self.hroi = self.plotx.plot()
        self.dockx.addWidget(self.plotx)
        self.ploty = pg.PlotWidget(self.docky)
        self.vroi = self.ploty.plot()
        self.docky.addWidget(self.ploty)
        layout = QtWidgets.QVBoxLayout()
        layout.addWidget(self.docks)
        self.setLayout(layout)
        xi = self.plotx.plotItem
        yi = self.ploty.plotItem
        self.proxyx = pg.SignalProxy(xi.scene().sigMouseMoved, rateLimit=60,
                                     slot=lambda e: self.mouseMoved1D(xi, e[0], self.sigXMouseMoved))
        self.proxyy = pg.SignalProxy(yi.scene().sigMouseMoved, rateLimit=60,
                                     slot=lambda e: self.mouseMoved1D(yi, e[0], self.sigYMouseMoved))

    def mouseMoved1D(self, plotItem, pos, signal):
        if plotItem.sceneBoundingRect().contains(pos):
            mousePoint = plotItem.vb.mapSceneToView(pos)
            signal.emit(mousePoint.x(), mousePoint.y())

    def saveSettings(self):
        s = QtCore.QSettings()
        s.setValue('CutSliceWidget/dockState', json.dumps(self.docks.saveState()))

    def loadSettings(self):
        s = QtCore.QSettings()
        dockState = s.value('CutSliceWidget/dockState', '', str)
        if dockState:
            # noinspection PyBroadException
            try:
                self.docks.restoreState(json.loads(dockState))
            except Exception:  # stupid!
                pass

    def showImage(self, array, adjust=False):
        adjust = adjust or self.plot2DView.image is None
        self.plot2DView.setImage(array, autoLevels=False, autoRange=False)
        if adjust:
            self.actionAdjustLevels.activate(QtWidgets.QAction.Trigger)

    def adjustImageLevels(self):
        if self.plot2DView.image is not None:
            self.plot2DView.autoRange()
            self.plot2DView.autoLevels()

    def plotHCut(self, x: np.ndarray, y: np.ndarray, **kwargs):
        self.hroi.setData(x, y, **kwargs)

    def plotVCut(self, x: np.ndarray, y: np.ndarray, **kwargs):
        self.vroi.setData(y, x, **kwargs)

    def setHCutSize(self, size):
        self.plot2DView.hsize = size

    def setVCutSize(self, size):
        self.plot2DView.vsize = size

    def setSmooth(self, smooth):
        self.plot2DView.setSmooth(smooth)

    def moveCross(self):
        self.plot2DView.moveCross()

    def fixCross(self):
        self.plot2DView.fixCross()

    def clear(self):
        self.plot2DView.setImage(None)
        self.ploty.plotItem.clear()
        self.plotx.plotItem.clear()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from PyQt5 import QtCore, QtWidgets, QtGui


class FixedWidget(QtWidgets.QWidget):
    def _fixWindow(self):
        self.resize(0, 0)
        geometry = self.geometry()
        self.setFixedSize(geometry.width(), geometry.height())

    def fixWindow(self):
        QtCore.QTimer.singleShot(0, self._fixWindow)

    def keyPressEvent(self, event: QtGui.QKeyEvent):
        key = event.key()
        if key == QtCore.Qt.Key_Escape:
            event.ignore()
        else:
            super().keyPressEvent(event)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import numpy as np
from PyQt5 import QtCore, QtGui, QtWidgets
import pyqtgraph as pg
from .axisitem import AxisItem
from .viewbox import ViewBox


class ImageView(pg.GraphicsLayoutWidget):
    sigHROIData = QtCore.pyqtSignal(np.ndarray, np.ndarray)
    sigVROIData = QtCore.pyqtSignal(np.ndarray, np.ndarray)
    sigHCutValues = QtCore.pyqtSignal(np.ndarray, np.ndarray)
    sigVCutValues = QtCore.pyqtSignal(np.ndarray, np.ndarray)
    sigMouseLeftClicked = QtCore.pyqtSignal(float, float)
    sigLeftAxisRangeChanged = QtCore.pyqtSignal(float, float)
    sigBottomAxisRangeChanged = QtCore.pyqtSignal(float, float)

    def __init__(self, parent=None, **kwargs):
        super().__init__(parent, **kwargs)
        self.move_cross = True
        self.image = None
        self.smooth = True
        self.old_image = None
        self.vsize = 1
        self.hsize = 1
        self.x = 0
        self.y = 0
        self.stepY = 0
        self.stepX = 0
        self.view = ViewBox()
        self.view.setAspectLocked(True)
        # TODO: .addPlot adds a PlotItem, which takes AxisItem parameters
        # TODO: this AxisItems should be considered to show angles from model (or momentum)
        self.leftAxis = AxisItem('left')
        self.bottomAxis = AxisItem('bottom')
        self.leftAxis.linkToView(self.view)
        self.bottomAxis.linkToView(self.view)
        axisItems = {'left': self.leftAxis, 'bottom': self.bottomAxis}
        self.plot2D = self.addPlot(viewBox=self.view, axisItems=axisItems)
        self.imageItem = pg.ImageItem()
        self.plot2D.addItem(self.imageItem)
        self.histogram = pg.HistogramLUTItem()
        self.histogram.setImageItem(self.imageItem)
        self.addItem(self.histogram)
        color = QtGui.QColor(255, 0, 0, 150)
        self.areaX = QtWidgets.QGraphicsRectItem(0, 0, 1, 1)
        self.areaX.setPen(QtGui.QPen(color))
        self.areaX.setBrush(QtGui.QBrush(color))
        self.areaY = QtWidgets.QGraphicsRectItem(0, 0, 1, 1)
        self.areaY.setPen(QtGui.QPen(color))
        self.areaY.setBrush(QtGui.QBrush(color))
        self.areaX.setVisible(False)
        self.areaY.setVisible(False)
        self.view.addItem(self.areaX)
        self.view.addItem(self.areaY)
        self.drawCrossHair()
        self.connectSignals()

    def connectSignals(self):
        self.sigHROIData.connect(self.calcHCut)
        self.sigVROIData.connect(self.calcVCut)
        self.view.sigMouseLeftClick.connect(lambda: self.sigMouseLeftClicked.emit(self.x, self.y))
        self.leftAxis.sigChangeRange.connect(self.sigLeftAxisRangeChanged.emit)
        self.bottomAxis.sigChangeRange.connect(self.sigBottomAxisRangeChanged.emit)

    def setImage(self, image, autoLevels=False, autoRange=False):
        self.image = image
        if self.image is None:
            self.imageItem.clear()
        else:
            self.imageItem.setImage(self.image, autoLevels=autoLevels, autoRange=autoRange)

    def setLevels(self, lmin, lmax):
        self.histogram.setLevels(lmin, lmax)

    def autoLevels(self, method='mean'):
        if method == 'mean':
            self.setLevels(0, self.image.mean() + 8 * self.image.std())
        elif method == 'max':
            levelMax = self.quickMinMax(self.image)[1]
            self.setLevels(-levelMax / 20, levelMax / 4)

    def autoRange(self):
        self.view.autoRange()

    def drawCrossHair(self):
        self.vLine = pg.InfiniteLine(angle=90, movable=False)
        self.hLine = pg.InfiniteLine(angle=0, movable=False)
        self.view.addItem(self.vLine, ignoreBounds=True)
        self.view.addItem(self.hLine, ignoreBounds=True)
        self.proxy = pg.SignalProxy(self.view.scene().sigMouseMoved, rateLimit=60, slot=self.mouseMoved)

    def mouseMoved(self, event=None):
        if self.image is None or event is None or not self.move_cross:
            return
        self.drawROI(event[0])
        self.calcROI()

    def drawROI(self, pos):
        if self.view.sceneBoundingRect().contains(pos):
            mousePoint = self.view.mapSceneToView(pos)
            self.x, self.y = mousePoint.x(), mousePoint.y()
            self.vLine.setPos(self.x)
            self.hLine.setPos(self.y)

    def calcROI(self):
        (xmin, xmax), (ymin, ymax) = self.view.viewRange()
        hpos = xmin, self.y - self.hsize / 2
        hsize = xmax - xmin, self.hsize
        vpos = self.x - self.vsize / 2, ymin
        vsize = self.vsize, ymax - ymin
        if not self.smooth:
            hpos, hsize, vpos, vsize = map(np.rint, (hpos, hsize, vpos, vsize))
        self.sigHROIData.emit(*self.getROIData(hpos, hsize))
        self.sigVROIData.emit(*self.getROIData(vpos, vsize))

    def getROIData(self, pos, size):
        roi = pg.ROI(pos=pos, size=size, angle=0)
        roi.setVisible(False)
        self.view.addItem(roi)
        # we don't want to calc a cut if the log mode is on
        image = self.image if self.old_image is None else self.old_image
        data, coords = roi.getArrayRegion(image, self.imageItem, returnMappedCoords=True)
        self.view.removeItem(roi)
        return data, coords

    def calcHCoords(self, data, coords):
        if data is None:
            return
        while data.ndim > 1:
            data = data.mean(axis=1)
        while coords.ndim > 2:
            coords = coords[:, :, 0]
        hcd = coords - coords[:, 0, np.newaxis]
        x = np.sqrt((hcd ** 2).sum(axis=0)) + coords[0][0]
        return x, data

    def calcHCut(self, data, coords):
        try:
            x, data = self.calcHCoords(data, coords)
        except IndexError:
            return
        self.sigHCutValues.emit(x, data)

    def calcVCoords(self, data, coords):
        if data is None:
            return
        while data.ndim > 1:
            data = data.mean(axis=0)
        while coords.ndim > 2:
            coords = coords[:, 0, :]
        hcd = coords - coords[:, 0, np.newaxis]
        y = np.sqrt((hcd ** 2).sum(axis=0)) + coords[1][0]
        return y, data

    def calcVCut(self, data, coords):
        try:
            y, data = self.calcVCoords(data, coords)
        except IndexError:
            return
        self.sigVCutValues.emit(y, data)

    def rotateImage(self):
        if self.image is None:
            return
        self.setImage(np.rot90(self.image, 3))

    def flipHImage(self):
        if self.image is None:
            return
        self.setImage(np.flipud(self.image))

    def flipVImage(self):
        if self.image is None:
            return
        self.setImage(np.fliplr(self.image))

    def logImage(self, on: bool):
        if self.image is None:
            return
        if on:
            self.old_image = self.image
            # there is no point to log array values which are less than 1
            # it results in NaNs and zero divisions, too bad
            self.setImage(np.log(np.where(self.image > 1, self.image, np.ones_like(self.image))), True, True)
        else:
            image = self.old_image
            self.old_image = None
            self.setImage(image, True)

    def setSmooth(self, smooth):
        self.smooth = smooth

    def rescaleImage(self):
        if self.image is None:
            return
        viewRect = self.view.rect()
        self.imageItem.setRect(viewRect)
        self.view.autoRange()
        width, height = viewRect.width(), viewRect.height()
        self.stepY = height / self.imageItem.height()
        self.stepX = width / self.imageItem.width()
        self.areaY.setRect(0, self.stepY, width, self.stepY + 1)
        self.areaX.setRect(self.stepX, 0, self.stepX + 1, height)

    def quickMinMax(self, data):
        """
        Estimate the min/max values of *data* by subsampling.
        """
        while data.size > 1e6:
            ax = np.argmax(data.shape)
            sl = [slice(None)] * data.ndim
            sl[ax] = slice(None, None, 2)
            data = data[sl]
        return np.nanmin(data), np.nanmax(data)

    def moveCross(self):
        self.move_cross = True

    def fixCross(self):
        self.move_cross = False



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from PyQt5 import QtCore
import pyqtgraph as pg


class ViewBox(pg.ViewBox):
    sigMouseLeftClick = QtCore.pyqtSignal()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.connectSignals()

    def connectSignals(self):
        pass

    def mouseClickEvent(self, event):
        if event.button() == QtCore.Qt.LeftButton:
            self.sigMouseLeftClick.emit()
        else:
            super().mouseClickEvent(event)
